! See copyright notice in the COPYRIGHT file.
program mus_cm_to_pdf_test
  use env_module,         only: rk, eps
  use tem_general_module, only: tem_start, tem_finalize, tem_general_type
  use mus_d3q27_module,   only: central_moment_split, cm_to_pdf

  implicit none

  logical :: error
  real(kind=rk) :: tolerance

  type( tem_general_type ) :: general
  integer :: ii, jj, kk
  real(kind=rk) ::  f(-1:1,-1:1,-1:1) = -100.0_rk
  real(kind=rk) :: ff(-1:1,-1:1,-1:1) = -234.0_rk
  real(kind=rk) :: k(0:2,0:2,0:2) = -123.0_rk
  real(kind=rk) :: ux, uy, uz, rho, inv_rho
  real(kind=rk) :: err_sum

  call tem_start( 'CM to PDF Test', 'utest', general )
  error = .false.
  tolerance = eps * 2500._rk
  write(*,"(A,F20.14)") 'tolerance = ', tolerance

  ! init PDF
  write(*,"(A)") 'Initialize PDF'
  do kk = -1, 1
    do jj = -1, 1
      do ii = -1, 1
        f( ii, jj, kk ) = abs(0.1d0 * ii + 0.2d0 * jj + 0.3d0 * kk * kk)
      end do
    end do
  end do
  f = f / sum(f)
  do kk = -1, 1
    do jj = -1, 1
      do ii = -1, 1
        write(*,"(A,3I3,A,F6.3)") 'f(',ii,jj,kk,') = ', f( ii, jj, kk )
      end do
    end do
  end do
  ! init PDF

  ! calc velocity
  rho = sum( f )
  inv_rho = 1.0_rk / rho
  ux  = ( sum(f(1,:,:)) - sum(f(-1, :, :)) ) * inv_rho
  uy  = ( sum(f(:,1,:)) - sum(f( :,-1, :)) ) * inv_rho
  uz  = ( sum(f(:,:,1)) - sum(f( :, :,-1)) ) * inv_rho
  write(*,"(A,F6.3)") 'rho = ', rho
  write(*,"(A,F6.3)") 'ux = ', ux
  write(*,"(A,F6.3)") 'uy = ', uy
  write(*,"(A,F6.3)") 'uz = ', uz

  err_sum = 0.0d0
  ! compare two ways of central moments calculation
  do kk = 0,2
    do jj = 0,2
      do ii = 0,2
        k(ii,jj,kk) = central_moment_split(f, ii,jj,kk, ux, uy, uz)
      end do
    end do
  end do
  ff = cm_to_pdf( k, ux, uy, uz )

        write(*,"(3A3,2A15  )") 'ii','jj','kk','f','ff'
  do kk = -1, 1
    do jj = -1, 1
      do ii = -1, 1
        write(*,"(3I3,2F15.11)") ii,jj,kk, f(ii,jj,kk), ff(ii,jj,kk)
      end do
    end do
  end do

  err_sum = sum( abs(f-ff) )
  if ( err_sum < tolerance ) then
    error = .false.
  else
    error = .true.
  end if

  call tem_finalize(general)
  if (error) then
    write(*,"(A,F18.14)") 'tolerance: ', tolerance
    write(*,"(A,F18.14)") '    error: ', err_sum
    write(*,"(A)") 'FAILED'
  else
    write(*,'(A)') 'PASSED'
  end if

contains

end program mus_cm_to_pdf_test
